<?php
	# Demo of Fax2 API, https://www.fax2.com.au/api
	# 
	# Set the variables below as appropriate:
	$USERNAME = "YOUR_USERNAME";
	$PASSWORD = "YOUR_PASSWORD";
	$DEST_NUMBER = "61280763351";

	# Then run the script from the command line:
	# $ php fax2-api-demo.php
	#
	# See also the README.md file
	
	use \Exception;

	class Client {
		static protected $URL_BASE = "https://api.fax2.com.au/v1";
	
		protected $token;
	
		function __construct() {
			$this->base_url = getenv("FAX2_API_BASE");
			if(!$this->base_url) $this->base_url = self::$URL_BASE;
			$this->token = null;
		}

		protected function new_client($endpoint) {
			$client = curl_init($this->base_url."/$endpoint");
			curl_setopt($client,CURLOPT_SAFE_UPLOAD,false);
			curl_setopt($client,CURLOPT_RETURNTRANSFER,true);
			return $client;
		}
		
		protected function authed_client($endpoint) {
			$client = $this->new_client($endpoint);
			curl_setopt($client,CURLOPT_HTTPHEADER,["Authorization: bearer $this->token"]);
			return $client;
		}
		
		protected function run_and_check($client) {
			$resp = curl_exec($client);
			$code = curl_getinfo($client,CURLINFO_HTTP_CODE);
			if(floor($code/100) != 2)
				throw new Exception("Status code $code from Fax2 srvice");
			$data = json_decode($resp);
			
			if(isset($data->{"error"})) {
				if(isset($data->{"error_description"}))
					throw new Exception(
						"Error from Fax2 service: {$data->{"error_description"}} ({$data->{"error"}})"
					);
				throw new Exception("Error from Fax2 service ({$data->{"error"}})");
			}
			curl_close($client);
			return $data;
		}
		
		function login($username,$password) {
			$client = $this->new_client("oauth2/token");
			curl_setopt($client,CURLOPT_USERPWD,"$username:$password");
			curl_setopt($client,CURLOPT_POSTFIELDS,
				http_build_query(["grant_type" => "client_credentials"])
			);
			$data = $this->run_and_check($client);
			$this->token = $data->{"access_token"};
		}
		
		function upload_text($text) {
			# This would use authed_client() but the curl module doesn't appear
			# to have a way to add headers without clobbering already-added
			# headers, so we have to set them all in one call.
			$client = $this->new_client("upload_document");
			curl_setopt($client,CURLOPT_HTTPHEADER,[
				"Authorization: bearer $this->token",
				"Content-Type: text/plain; charset=utf-8"
			]);
			curl_setopt($client,CURLOPT_POSTFIELDS,$text);
			$data = $this->run_and_check($client);
			return $data->{"document_id"};
		}
		
		function upload_file($content,$type) {
			$client = $this->new_client("upload_document");
			curl_setopt($client,CURLOPT_HTTPHEADER,[
				"Authorization: bearer $this->token",
				"Content-Type: $type"
			]);
			curl_setopt($client,CURLOPT_POSTFIELDS,$content);
			$data = $this->run_and_check($client);
			return $data->{"document_id"};
		}
		
		function send_fax($dest,$documents) {
			$client = $this->authed_client("send_fax");
			curl_setopt($client,CURLOPT_POST,true);
			
			$params = [];
			foreach($documents as $doc)
				$params[] = http_build_query(["documents[]" => $doc]);
			$params[] = http_build_query(["dest_number" => $dest]);
			curl_setopt($client,CURLOPT_POSTFIELDS,implode($params,"&"));
			$data = $this->run_and_check($client);
			return $data->{"pages"};
		}
	}
	
	function test() {
		$client = new Client;
		global $USERNAME,$PASSWORD,$DEST_NUMBER;
	
		$client->login($USERNAME,$PASSWORD);
		$doc_id1 = $client->upload_text("This is a sample cover page.");
		$doc_id2 = $client->upload_file(file_get_contents("sample.pdf"),"application/pdf");
		$pages = $client->send_fax($DEST_NUMBER,[$doc_id1,$doc_id2]);
		echo("Fax sent, total $pages page(s).\n");
	}
	
	test();
?>
